// This code has been modified to conform to conventions
// used in Edward Angel's book (Daniel Audet)

// Original source code can be found at: 
//     https://sites.google.com/site/webglbook/home/9-various-techniques

var canvas;
var gl;

// You can try a smaller offscreen framebuffer to observe aliasing
// var OFFSCREEN_WIDTH = 128, OFFSCREEN_HEIGHT = 128;
var OFFSCREEN_WIDTH = 2048, OFFSCREEN_HEIGHT = 2048;
var LIGHT_X = 0, LIGHT_Y = 7, LIGHT_Z = 2; // Position of the light source

// Coordinate transformation matrix
var global_modelMatrix = mat4();
var global_mvpMatrix = mat4();

window.onload = function init() {

    
  // Retrieve <canvas> element
    canvas = document.getElementById( "gl-canvas" );
    
    gl = WebGLUtils.setupWebGL( canvas );
    if ( !gl ) { alert( "WebGL isn't available" ); }

	
  // Initialize shaders for generating a shadow map
    var shadowProgram = initShaders( gl, "shadow-vertex-shader", "shadow-fragment-shader" );
    gl.useProgram( shadowProgram );

	// We add the variable locations as members of the shadowProgram object (allowed in Javascript)
  shadowProgram.vPositionLoc = gl.getAttribLocation(shadowProgram, 'vPosition');
  shadowProgram.MvpMatrixLoc = gl.getUniformLocation(shadowProgram, 'MvpMatrix');
  if (shadowProgram.vPositionLoc < 0 || !shadowProgram.MvpMatrixLoc) {
    console.log('Failed to get the storage location of attribute or uniform variable from shadowProgram'); 
    return;
  }

  // Initialize shaders for regular drawing
    var regularProgram = initShaders( gl, "normal-vertex-shader", "normal-fragment-shader" );
    gl.useProgram( regularProgram );
	
	// We add the variable locations as members of the regularProgram object (allowed in Javascript)
  regularProgram.vPositionLoc = gl.getAttribLocation(regularProgram, 'vPosition');
  regularProgram.vColorLoc = gl.getAttribLocation(regularProgram, 'vColor');
  regularProgram.MvpMatrixLoc = gl.getUniformLocation(regularProgram, 'MvpMatrix');
  regularProgram.MvpMatrixFromLightLoc = gl.getUniformLocation(regularProgram, 'MvpMatrixFromLight');
  regularProgram.ShadowMapLoc = gl.getUniformLocation(regularProgram, 'ShadowMap');
  if (regularProgram.vPositionLoc < 0 || regularProgram.vColorLoc < 0 || !regularProgram.MvpMatrixLoc ||
      !regularProgram.MvpMatrixFromLightLoc || !regularProgram.ShadowMapLoc) {
    console.log('Failed to get the storage location of attribute or uniform variable from regularProgram'); 
    return;
  }

  // Set the vertex information
  var triangle = initVertexBuffersForTriangle(gl);
  var plane = initVertexBuffersForPlane(gl);
  if (!triangle || !plane) {
    console.log('Failed to set the vertex information');
    return;
  }

  // Initialize framebuffer object (FBO)  
  var fbo = initFramebufferObject(gl);
  if (!fbo) {
    console.log('Failed to initialize frame buffer object');
    return;
  }
  gl.activeTexture(gl.TEXTURE0); // Set a texture object to the texture unit
  gl.bindTexture(gl.TEXTURE_2D, fbo.texture);

  // Set the clear color and enable the depth test
  gl.clearColor(0, 0, 0, 1);
  gl.enable(gl.DEPTH_TEST);
  

  var viewProjMatrixFromLight = mat4();
  viewProjMatrixFromLight = mult(viewProjMatrixFromLight, perspective(70.0, 1.0, 1.0, 100.0));
  var eye = [LIGHT_X, LIGHT_Y, LIGHT_Z];
  var at = [0, 0, 0];
  var up = [0, 1, 0];
  viewProjMatrixFromLight = mult(viewProjMatrixFromLight, lookAt(eye, at, up));
  

  var viewProjMatrix = mat4();
  viewProjMatrix =  mult(viewProjMatrix, perspective(45, canvas.width/canvas.height, 1.0, 100.0));
  var eye = [0.0, 7.0, 9.0];
  var at = [0, 0, 0];
  var up = [0, 1, 0];
  viewProjMatrix =  mult(viewProjMatrix, lookAt(eye, at, up));
  
  var currentAngle = 0.0; // Current rotation angle (degrees)
  var mvpMatrixFromLight_t = mat4(); // A model view projection matrix from light source (for triangle)
  var mvpMatrixFromLight_p = mat4(); // A model view projection matrix from light source (for plane)
  
 var render = function() {
    currentAngle = animate(currentAngle);

// Draw in the offscreen framebuffer	
	
    gl.bindFramebuffer(gl.FRAMEBUFFER, fbo);               // Change the drawing destination to FBO
    gl.viewport(0, 0, OFFSCREEN_HEIGHT, OFFSCREEN_HEIGHT); // Set view port for FBO
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);   // Clear FBO    

    gl.useProgram(shadowProgram); // Set shaders for generating a shadow map

// Draw triangle
    global_modelMatrix = rotate(currentAngle, 0, 1, 0);

	initAttributeVariable(gl, shadowProgram.vPositionLoc, triangle.vertexBuffer);
	if (shadowProgram.vColorLoc != undefined) // If a_Color is defined to attribute
        initAttributeVariable(gl, shadowProgram.vColorLoc, triangle.colorBuffer);

    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, triangle.indexBuffer);

  // Calculate the model view project matrix and pass it to MvpMatrix
    global_mvpMatrix = viewProjMatrixFromLight;
    global_mvpMatrix = mult(global_mvpMatrix, global_modelMatrix);  //  #########
    gl.uniformMatrix4fv(shadowProgram.MvpMatrixLoc, false, flatten(global_mvpMatrix));

    gl.drawElements(gl.TRIANGLES, triangle.numIndices, gl.UNSIGNED_BYTE, 0);
	
    mvpMatrixFromLight_t = global_mvpMatrix; // Used later
 
// Draw plane
//    global_modelMatrix = rotate(-45, 0, 1, 1);
    global_modelMatrix = rotate(currentAngle, 1, 1, 0);

	initAttributeVariable(gl, shadowProgram.vPositionLoc, plane.vertexBuffer);
	if (shadowProgram.vColorLoc != undefined) // If a_Color is defined to attribute
        initAttributeVariable(gl, shadowProgram.vColorLoc, plane.colorBuffer);

    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, plane.indexBuffer);

  // Calculate the model view project matrix and pass it to MvpMatrix
    global_mvpMatrix = viewProjMatrixFromLight;
    global_mvpMatrix = mult(global_mvpMatrix, global_modelMatrix);  //  #########
    gl.uniformMatrix4fv(shadowProgram.MvpMatrixLoc, false, flatten(global_mvpMatrix));

    gl.drawElements(gl.TRIANGLES, plane.numIndices, gl.UNSIGNED_BYTE, 0);

    mvpMatrixFromLight_p = global_mvpMatrix; // Used later

	
	
	
// Draw in the regular framebuffer
	
    gl.bindFramebuffer(gl.FRAMEBUFFER, null);               // Change the drawing destination to color buffer
    gl.viewport(0, 0, canvas.width, canvas.height);
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);    // Clear color and depth buffer

    gl.useProgram(regularProgram); // Set the shader for regular drawing
    gl.uniform1i(regularProgram.ShadowMapLoc, 0);  // Pass 0 because gl.TEXTURE0 is enabled	

// Draw triangle
    gl.uniformMatrix4fv(regularProgram.MvpMatrixFromLightLoc, false, flatten(mvpMatrixFromLight_t));

    global_modelMatrix = rotate(currentAngle, 0, 1, 0);

	initAttributeVariable(gl, regularProgram.vPositionLoc, triangle.vertexBuffer);
	if (regularProgram.vColorLoc != undefined) // If a_Color is defined to attribute
        initAttributeVariable(gl, regularProgram.vColorLoc, triangle.colorBuffer);

    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, triangle.indexBuffer);

  // Calculate the model view project matrix and pass it to MvpMatrix
    global_mvpMatrix = viewProjMatrix;
    global_mvpMatrix = mult(global_mvpMatrix, global_modelMatrix);  //  #########
    gl.uniformMatrix4fv(regularProgram.MvpMatrixLoc, false, flatten(global_mvpMatrix));

    gl.drawElements(gl.TRIANGLES, triangle.numIndices, gl.UNSIGNED_BYTE, 0);

// Draw plane
    gl.uniformMatrix4fv(regularProgram.MvpMatrixFromLightLoc, false, flatten(mvpMatrixFromLight_p));

//    global_modelMatrix = rotate(-45, 0, 1, 1);
    global_modelMatrix = rotate(currentAngle, 1, 1, 0);

	initAttributeVariable(gl, regularProgram.vPositionLoc, plane.vertexBuffer);
	if (regularProgram.vColorLoc != undefined) // If a_Color is defined to attribute
        initAttributeVariable(gl, regularProgram.vColorLoc, plane.colorBuffer);

    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, plane.indexBuffer);

  // Calculate the model view project matrix and pass it to MvpMatrix
    global_mvpMatrix = viewProjMatrix;
    global_mvpMatrix = mult(global_mvpMatrix, global_modelMatrix);  //  #########
    gl.uniformMatrix4fv(regularProgram.MvpMatrixLoc, false, flatten(global_mvpMatrix));

    gl.drawElements(gl.TRIANGLES, plane.numIndices, gl.UNSIGNED_BYTE, 0);

    window.requestAnimationFrame(render, canvas);
  };
  
   render(); 
}


  

// Assign the buffer objects and enable the assignment
function initAttributeVariable(gl, attributeLoc, buffer) {
  gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
  gl.vertexAttribPointer(attributeLoc, buffer.num, buffer.type, false, 0, 0);
  gl.enableVertexAttribArray(attributeLoc);
}

function initVertexBuffersForPlane(gl) {
  // Create a plane
  //  v1------v0
  //  |        | 
  //  |        |
  //  |        |
  //  v2------v3

  // Vertex coordinates
  var vertices = new Float32Array([
    3.0, -1.7, 2.5,  -3.0, -1.7, 2.5,  -3.0, -1.7, -2.5,   3.0, -1.7, -2.5    // v0-v1-v2-v3
  ]);

  // Colors
  var colors = new Float32Array([
    1.0, 1.0, 1.0,    1.0, 1.0, 1.0,  1.0, 1.0, 1.0,   1.0, 1.0, 1.0
  ]);

  // Indices of the vertices
  var indices = new Uint8Array([0, 1, 2,   0, 2, 3]);

  var o = new Object(); // Utilize Object object to return multiple buffer objects together

  // Write vertex information to buffer object
  o.vertexBuffer = initArrayBufferForLaterUse(gl, vertices, 3, gl.FLOAT);
  o.colorBuffer = initArrayBufferForLaterUse(gl, colors, 3, gl.FLOAT);
  o.indexBuffer = initElementArrayBufferForLaterUse(gl, indices, gl.UNSIGNED_BYTE);
  if (!o.vertexBuffer || !o.colorBuffer || !o.indexBuffer) return null; 

  o.numIndices = indices.length;

  // Unbind the buffer object
  gl.bindBuffer(gl.ARRAY_BUFFER, null);
  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);

  return o;
}

function initVertexBuffersForTriangle(gl) {
  // Create a triangle
  //       v2
  //      / | 
  //     /  |
  //    /   |
  //  v0----v1

  // Vertex coordinates
  var vertices = new Float32Array([-0.8, 3.5, 0.0,  0.8, 3.5, 0.0,  0.0, 3.5, 1.8]);
  // Colors
  var colors = new Float32Array([1.0, 0.5, 0.0,  1.0, 0.5, 0.0,  1.0, 0.0, 0.0]);    
  // Indices of the vertices
  var indices = new Uint8Array([0, 1, 2]);

  var o = new Object();  // Utilize Object object to return multiple buffer objects together

  // Write vertex information to buffer object
  o.vertexBuffer = initArrayBufferForLaterUse(gl, vertices, 3, gl.FLOAT);
  o.colorBuffer = initArrayBufferForLaterUse(gl, colors, 3, gl.FLOAT);
  o.indexBuffer = initElementArrayBufferForLaterUse(gl, indices, gl.UNSIGNED_BYTE);
  if (!o.vertexBuffer || !o.colorBuffer || !o.indexBuffer) return null; 

  o.numIndices = indices.length;

  // Unbind the buffer object
  gl.bindBuffer(gl.ARRAY_BUFFER, null);
  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);

  return o;
}

function initArrayBufferForLaterUse(gl, data, num, type) {
  // Create a buffer object
  var buffer = gl.createBuffer();
  if (!buffer) {
    console.log('Failed to create the buffer object');
    return null;
  }
  // Write date into the buffer object
  gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
  gl.bufferData(gl.ARRAY_BUFFER, data, gl.STATIC_DRAW);

  // Store the necessary information to assign the object to the attribute variable later
  buffer.num = num;
  buffer.type = type;

  return buffer;
}

function initElementArrayBufferForLaterUse(gl, data, type) {
  // Create a buffer object
  var buffer = gl.createBuffer();
  if (!buffer) {
    console.log('Failed to create the buffer object');
    return null;
  }
  // Write date into the buffer object
  gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, buffer);
  gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, data, gl.STATIC_DRAW);

  buffer.type = type;

  return buffer;
}

function initFramebufferObject(gl) {
  var framebuffer, texture, depthBuffer;

  // Define the error handling function
  var error = function() {
    if (framebuffer) gl.deleteFramebuffer(framebuffer);
    if (texture) gl.deleteTexture(texture);
    if (depthBuffer) gl.deleteRenderbuffer(depthBuffer);
    return null;
  }

  // Create a framebuffer object (FBO)
  framebuffer = gl.createFramebuffer();
  if (!framebuffer) {
    console.log('Failed to create frame buffer object');
    return error();
  }

  // Create a texture object and set its size and parameters
  texture = gl.createTexture(); // Create a texture object
  if (!texture) {
    console.log('Failed to create texture object');
    return error();
  }
  gl.bindTexture(gl.TEXTURE_2D, texture);
  gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, OFFSCREEN_WIDTH, OFFSCREEN_HEIGHT, 0, gl.RGBA, gl.UNSIGNED_BYTE, null);
  gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);

  // Create a renderbuffer object and Set its size and parameters
  depthBuffer = gl.createRenderbuffer(); // Create a renderbuffer object
  if (!depthBuffer) {
    console.log('Failed to create renderbuffer object');
    return error();
  }
  gl.bindRenderbuffer(gl.RENDERBUFFER, depthBuffer);
  gl.renderbufferStorage(gl.RENDERBUFFER, gl.DEPTH_COMPONENT16, OFFSCREEN_WIDTH, OFFSCREEN_HEIGHT);

  // Attach the texture and the renderbuffer object to the FBO
  gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);
  gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
  gl.framebufferRenderbuffer(gl.FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.RENDERBUFFER, depthBuffer);

  // Check if FBO is configured correctly
  var e = gl.checkFramebufferStatus(gl.FRAMEBUFFER);
  if (gl.FRAMEBUFFER_COMPLETE !== e) {
    console.log('Frame buffer object is incomplete: ' + e.toString());
    return error();
  }

  framebuffer.texture = texture; // keep the required object

  // Unbind the buffer object
  gl.bindFramebuffer(gl.FRAMEBUFFER, null);
  gl.bindTexture(gl.TEXTURE_2D, null);
  gl.bindRenderbuffer(gl.RENDERBUFFER, null);

  return framebuffer;
}

var ANGLE_STEP = 40;   // The increments of rotation angle (degrees)

var last = Date.now(); // Last time that this function was called

function animate(angle) {
  var now = Date.now();   // Calculate the elapsed time
  var elapsed = now - last;
  last = now;
  // Update the current rotation angle (adjusted by the elapsed time)
  var newAngle = angle + (ANGLE_STEP * elapsed) / 1000.0;
  return newAngle % 360;
}
