// This shadow mapping program was developped by Daniel Audet and uses sections of code  
// from http://math.hws.edu/eck/cs424/notes2013/19_GLSL.html
//
//  It has been adapted to be compatible with the "MV.js" library developped
//  for the book "Interactive Computer Graphics" by Edward Angel and Dave Shreiner.
//

"use strict";

var gl;   // The webgl context.


var projection;   //--- projection matrix
var modelview;    // modelview matrix
var flattenedmodelview;    //--- flattened modelview matrix

var normalmatrix = mat3();  //--- create a 3X3 matrix that will affect normals

var rotator;   // A SimpleRotator object to enable rotation by mouse dragging.

var sphere, cylinder, box, teapot, disk, torus, cone;  // model identifiers

var regularprogram;  // "regular" shader program identifier

//  ######  Section added to allow shadow mapping (beginning)

var canvas;
var shadowprogram;

// Coordinate transformation matrices
var global_modelMatrix = mat4();
var global_mvpMatrix = mat4();
var ProjectionMatrixFromLight, ModelviewMatrixFromLight;
var ProjectionMatrix, ModelviewMatrix;
var matrixfromlight = new Object();

var fbo;

var OFFSCREEN_WIDTH = 8192, OFFSCREEN_HEIGHT = 8192;
var LIGHT_X = 20.0, LIGHT_Y = 20.0, LIGHT_Z = 50.0; // Position of the light source
var initialmodelview;

var xdisplacement = 0;
var ydisplacement = 0;

//  ######  End of section added to allow shadow mapping

var lightposition = vec4(LIGHT_X, LIGHT_Y, LIGHT_Z, 1.0);

var lightAmbient = vec4(1.0, 1.0, 1.0, 1.0);
var lightDiffuse = vec4(1.0, 1.0, 1.0, 1.0);
var lightSpecular = vec4(1.0, 1.0, 1.0, 1.0);

var materialAmbient = vec4(0.0, 0.1, 0.3, 1.0);
var materialDiffuse = vec4(0.48, 0.55, 0.69, 1.0);
var materialSpecular = vec4(0.48, 0.55, 0.69, 1.0);
var materialShininess = 100.0;

var ambientProduct, diffuseProduct, specularProduct;

var showScene = 1;
var toggleStretchMap = 0;

window.onload = function init() {
    try {
        canvas = document.getElementById("glcanvas");
        gl = canvas.getContext("webgl");
        if (!gl) {
            gl = canvas.getContext("experimental-webgl");
        }
        if (!gl) {
            throw "Could not create WebGL context.";
        }

//  ######  Section added to allow shadow mapping (beginning)
  // Initialize shaders for generating a shadow map
    shadowprogram = initShaders( gl, "shadow-vertex-shader", "shadow-fragment-shader" );
    gl.useProgram( shadowprogram );

	// We add the variable locations as members of the shadowprogram object (allowed in Javascript)
    shadowprogram.vcoordsLoc = gl.getAttribLocation(shadowprogram, 'vcoords');
    shadowprogram.vnormalLoc = gl.getAttribLocation(shadowprogram, 'vnormal');
    shadowprogram.vtexcoordLoc = gl.getAttribLocation(shadowprogram, 'vtexcoord');
    shadowprogram.modelviewLoc = gl.getUniformLocation(shadowprogram, 'modelview');
    shadowprogram.projectionLoc = gl.getUniformLocation(shadowprogram, 'projection');
    shadowprogram.normalmatrixLoc = gl.getUniformLocation(shadowprogram, 'normalmatrix');
    if (shadowprogram.vcoordsLoc < 0 || !shadowprogram.modelviewLoc || !shadowprogram.projectionLoc) {
      console.log('Failed to get the storage location of attribute or uniform variable from shadowprogram'); 
      return;
    }

		 // Initialize framebuffer object (FBO)  
    fbo = initFramebufferObject(gl);
    if (!fbo) {
      console.log('Failed to initialize frame buffer object');
      return;
    }
    gl.activeTexture(gl.TEXTURE0); // Set a texture object to the texture unit
    gl.bindTexture(gl.TEXTURE_2D, fbo.texture);
  

    ProjectionMatrixFromLight = mat4();
    ProjectionMatrixFromLight = mult(ProjectionMatrixFromLight, perspective(60.0, 1.0, 1.0, 200.0));
    var eye = [LIGHT_X, LIGHT_Y, LIGHT_Z];
    var at = [0, 0, 0];
    var up = [0, 1, 0];
    ModelviewMatrixFromLight = mat4();
    ModelviewMatrixFromLight = mult(ModelviewMatrixFromLight, lookAt(eye, at, up));
  

    ProjectionMatrix = mat4();
    ProjectionMatrix =  mult(ProjectionMatrix, perspective(60, canvas.width/canvas.height, 1.0, 200.0));
    eye = [-7.0, -7.0, 50.0];
    at = [0, 0, 0];
    up = [0, 1, 0];
    ModelviewMatrix = mat4();
    ModelviewMatrix =  mult(ModelviewMatrix, lookAt(eye, at, up));
  
  
//  ######  End of section added to allow shadow mapping
		
    // LOAD SHADER (standard texture mapping)
    regularprogram = initShaders( gl, "vshader", "fshader" );

    gl.useProgram(regularprogram);

    // locate variables for further use
    regularprogram.vcoordsLoc = gl.getAttribLocation(regularprogram, "vcoords");
    regularprogram.vnormalLoc = gl.getAttribLocation(regularprogram, "vnormal");
    regularprogram.vtexcoordLoc = gl.getAttribLocation(regularprogram, "vtexcoord");

    regularprogram.modelviewLoc = gl.getUniformLocation(regularprogram, "modelview");
    regularprogram.projectionLoc = gl.getUniformLocation(regularprogram, "projection");
    regularprogram.normalmatrixLoc = gl.getUniformLocation(regularprogram, "normalmatrix");

    gl.enableVertexAttribArray(regularprogram.vcoordsLoc);
    gl.enableVertexAttribArray(regularprogram.vnormalLoc);
    gl.enableVertexAttribArray(regularprogram.vtexcoordLoc);

    gl.enable(gl.DEPTH_TEST);

	// You can use basic models using the following lines

    sphere = createModel(uvSphere(10.0, 25.0, 25.0));
    cylinder = createModel(uvCylinder(10.0, 20.0, 25.0, false, false));
    box = createModel(cube(10.0));

	teapot = createModel(teapotModel);
    disk = createModel(ring(5.0, 10.0, 25.0));
    torus = createModel(uvTorus(15.0, 5.0, 25.0, 25.0));
    cone = createModel(uvCone(10.0, 20.0, 25.0, true));
 
    ambientProduct = mult(lightAmbient, materialAmbient);
    diffuseProduct = mult(lightDiffuse, materialDiffuse);
    specularProduct = mult(lightSpecular, materialSpecular);
    }
    catch (e) {
        document.getElementById("message").innerHTML =
             "Could not initialize WebGL: " + e;
        return;
    }

    document.addEventListener("keydown", doKey, false);  // add a callback function (when a key is pressed)

	    // Associate buttons to functions
		
    document.getElementById("ButtonScene").onclick = function () {
	  showScene = 1;
	  gl.useProgram( shadowprogram );
	  gl.uniform1i(gl.getUniformLocation(shadowprogram, "showDepthMap"), 0);
      gl.uniform1i(gl.getUniformLocation(shadowprogram, "stretchDepthRange"), 0); 
	  toggleStretchMap = 0;
    };
	
    document.getElementById("ButtonDepthMap").onclick = function () {
	  showScene = 0;
	  gl.useProgram( shadowprogram );
	  gl.uniform1i(gl.getUniformLocation(shadowprogram, "showDepthMap"), 1);
      gl.uniform1i(gl.getUniformLocation(shadowprogram, "stretchDepthRange"), 0);
      toggleStretchMap = 0;	  
	};
	
    document.getElementById("ButtonRescale").onclick = function () {
	  if(showScene == 0){
	     if(toggleStretchMap == 0){
		    gl.useProgram( shadowprogram );
	        gl.uniform1i(gl.getUniformLocation(shadowprogram, "stretchDepthRange"), 1); 
	        toggleStretchMap = 1;
	     }
	     else{
	        toggleStretchMap = 0;
			gl.useProgram( shadowprogram );
	        gl.uniform1i(gl.getUniformLocation(shadowprogram, "stretchDepthRange"), 0);
	     }
	  }
	};

    setInterval(render, 1000);
}


function render() {

//  ######  Section added to allow shadow mapping (beginning)
// FIRST PASS
 if(showScene == 1){
    // Draw in the offscreen framebuffer  (to obtain the depth map)
    gl.bindFramebuffer(gl.FRAMEBUFFER, fbo);               // Change the drawing destination to FBO
    gl.viewport(0, 0, OFFSCREEN_HEIGHT, OFFSCREEN_HEIGHT); // Set view port for FBO
	}
 if(showScene == 0){
    // Draw in the screen color buffer  (to show the depth map)
    gl.bindFramebuffer(gl.FRAMEBUFFER, null);               // Change the drawing destination to color buffer
    gl.viewport(0, 0, canvas.width, canvas.height);
	}
	
    // Set the clear color 
    gl.clearColor(0, 0, 0, 1);
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);   // Clear FBO    

    gl.useProgram(shadowprogram); // Set shaders for generating a shadow map


    projection = ProjectionMatrixFromLight;
    modelview = ModelviewMatrixFromLight;
	normalmatrix = extractnormalmatrix(modelview);
		
    initialmodelview = modelview;

    gl.uniformMatrix4fv(shadowprogram.projectionLoc, false, flatten(projection));  // send projection matrix to the new shader program

	gl.enableVertexAttribArray(shadowprogram.vcoordsLoc);
	gl.disableVertexAttribArray(shadowprogram.vnormalLoc);
	gl.disableVertexAttribArray(shadowprogram.vtexcoordLoc);

	drawscene(shadowprogram);

// SECOND PASS
 if(showScene == 1){
    // Draw in the screen color buffer
    gl.bindFramebuffer(gl.FRAMEBUFFER, null);               // Change the drawing destination to color buffer
    gl.viewport(0, 0, canvas.width, canvas.height);
	}
 if(showScene == 0){
    gl.bindFramebuffer(gl.FRAMEBUFFER, fbo);               // Change the drawing destination to FBO
    gl.viewport(0, 0, OFFSCREEN_HEIGHT, OFFSCREEN_HEIGHT); // Set view port for FBO
    }
	
//  ######  End of section added to allow shadow mapping

    //  Select shader program 
    gl.useProgram(regularprogram);

    gl.clearColor(0.79, 0.76, 0.27, 1);
    gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

    projection = ProjectionMatrix;
	modelview = ModelviewMatrix;
	normalmatrix = extractnormalmatrix(modelview);
		
    initialmodelview = modelview;

    gl.uniform4fv(gl.getUniformLocation(regularprogram, "ambientProduct"), flatten(ambientProduct));
    gl.uniform4fv(gl.getUniformLocation(regularprogram, "diffuseProduct"), flatten(diffuseProduct));
    gl.uniform4fv(gl.getUniformLocation(regularprogram, "specularProduct"), flatten(specularProduct));
    gl.uniform1f(gl.getUniformLocation(regularprogram, "shininess"), materialShininess);

    gl.uniform4fv(gl.getUniformLocation(regularprogram, "lightposition"), flatten(lightposition));

	gl.uniform1i(gl.getUniformLocation(regularprogram, "ShadowMap"), 0);  // Pass 0 because gl.TEXTURE0 is enabled
		
    gl.uniformMatrix4fv(regularprogram.projectionLoc, false, flatten(projection));  // send projection matrix to the new shader program
	
	gl.enableVertexAttribArray(regularprogram.vcoordsLoc);
    gl.enableVertexAttribArray(regularprogram.vnormalLoc);
    gl.disableVertexAttribArray(regularprogram.vtexcoordLoc);  // we do not need texture coordinates

    drawscene(regularprogram);
	
	}

function drawscene(program) {
	
     // draw a background box
    modelview = initialmodelview;
    modelview = mult(modelview, translate(-6.0, -8.0, -60.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(8.0, 8.0, 0.1));
	if (program == shadowprogram){
	   matrixfromlight.boxbackground = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.boxbackground));   
	}
    box.render(program);


    //  now, draw sphere model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(15.0, 0.0, 0.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.5, 0.5, 0.5));
	if (program == shadowprogram){
	   matrixfromlight.sphere = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.sphere));   
	}
    sphere.render(program);


    //  now, draw box model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(10.0, 10.0, 0.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.5, 0.5, 0.5));
	if (program == shadowprogram){
	   matrixfromlight.box = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.box));   
	}
    box.render(program);
		
    //  now, draw cylinder model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(0.0, 15.0, 0.0));
    modelview = mult(modelview, rotate(90.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.3, 0.3, 0.3));
	if (program == shadowprogram){
	   matrixfromlight.cylinder = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.cylinder));   
	}
    cylinder.render(program);

    //  now, draw cone model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(-10.0, 10.0, 0.0));
    modelview = mult(modelview, rotate(-90.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.5, 0.5, 0.5));
	if (program == shadowprogram){
	   matrixfromlight.cone = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.cone));   
	}
    cone.render(program);
		
    //  now, draw torus model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(-15.0, -2.0, 0.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.3, 0.3, 0.3));
	if (program == shadowprogram){
	   matrixfromlight.torus = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.torus));   
	}
    torus.render(program);
		
    //  now, draw disk model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(-8.0, -12.0, 0.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.5, 0.5, 0.5));
	if (program == shadowprogram){
	   matrixfromlight.disk = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.disk));   
	}
    disk.render(program);

	//  now, draw teapot model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(7.0, -11.0, 0.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.5, 0.5, 0.5));
	if (program == shadowprogram){
	   matrixfromlight.teapot1 = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.teapot1));   
	}
    teapot.render(program);

	//  now, draw second teapot model
    modelview = initialmodelview;
    modelview = mult(modelview, translate(12.0+xdisplacement, 4.0+ydisplacement, 15.0));
    modelview = mult(modelview, rotate(0.0, 1, 0, 0));
    normalmatrix = extractnormalmatrix(modelview);  // always extract the normal matrix before scaling
    modelview = mult(modelview, scale(0.5, 0.5, 0.5));
	if (program == shadowprogram){
	   matrixfromlight.teapot2 = mult(projection, modelview);  // store matrix for next pass
	}
	if (program == regularprogram){
       gl.uniformMatrix4fv(gl.getUniformLocation(regularprogram, "matrixfromlight"), false, flatten(matrixfromlight.teapot2));   
	}
    teapot.render(program);
	
}

function unflatten(matrix) {
    var result = mat4();
    result[0][0] = matrix[0]; result[1][0] = matrix[1]; result[2][0] = matrix[2]; result[3][0] = matrix[3];
    result[0][1] = matrix[4]; result[1][1] = matrix[5]; result[2][1] = matrix[6]; result[3][1] = matrix[7];
    result[0][2] = matrix[8]; result[1][2] = matrix[9]; result[2][2] = matrix[10]; result[3][2] = matrix[11];
    result[0][3] = matrix[12]; result[1][3] = matrix[13]; result[2][3] = matrix[14]; result[3][3] = matrix[15];

    return result;
}

function extractnormalmatrix(matrix) { // This function computes the transpose of the inverse of 
    // the upperleft part (3X3) of the modelview matrix (see http://www.lighthouse3d.com/tutorials/glsl-tutorial/the-normal-matrix/ )

    var result = mat3();
    var upperleft = mat3();
    var tmp = mat3();

    upperleft[0][0] = matrix[0][0];  // if no scaling is performed, one can simply use the upper left
    upperleft[1][0] = matrix[1][0];  // part (3X3) of the modelview matrix
    upperleft[2][0] = matrix[2][0];

    upperleft[0][1] = matrix[0][1];
    upperleft[1][1] = matrix[1][1];
    upperleft[2][1] = matrix[2][1];

    upperleft[0][2] = matrix[0][2];
    upperleft[1][2] = matrix[1][2];
    upperleft[2][2] = matrix[2][2];

    tmp = matrixinvert(upperleft);
    result = transpose(tmp);

    return result;
}

function matrixinvert(matrix) {

    var result = mat3();

    var det = matrix[0][0] * (matrix[1][1] * matrix[2][2] - matrix[2][1] * matrix[1][2]) -
                 matrix[0][1] * (matrix[1][0] * matrix[2][2] - matrix[1][2] * matrix[2][0]) +
                 matrix[0][2] * (matrix[1][0] * matrix[2][1] - matrix[1][1] * matrix[2][0]);

    var invdet = 1 / det;

    // inverse of matrix m
    result[0][0] = (matrix[1][1] * matrix[2][2] - matrix[2][1] * matrix[1][2]) * invdet;
    result[0][1] = (matrix[0][2] * matrix[2][1] - matrix[0][1] * matrix[2][2]) * invdet;
    result[0][2] = (matrix[0][1] * matrix[1][2] - matrix[0][2] * matrix[1][1]) * invdet;
    result[1][0] = (matrix[1][2] * matrix[2][0] - matrix[1][0] * matrix[2][2]) * invdet;
    result[1][1] = (matrix[0][0] * matrix[2][2] - matrix[0][2] * matrix[2][0]) * invdet;
    result[1][2] = (matrix[1][0] * matrix[0][2] - matrix[0][0] * matrix[1][2]) * invdet;
    result[2][0] = (matrix[1][0] * matrix[2][1] - matrix[2][0] * matrix[1][1]) * invdet;
    result[2][1] = (matrix[2][0] * matrix[0][1] - matrix[0][0] * matrix[2][1]) * invdet;
    result[2][2] = (matrix[0][0] * matrix[1][1] - matrix[1][0] * matrix[0][1]) * invdet;

    return result;
}


function createModel(modelData) {
    var model = {};
    model.coordsBuffer = gl.createBuffer();
    model.normalBuffer = gl.createBuffer();
    model.textureBuffer = gl.createBuffer();
    model.indexBuffer = gl.createBuffer();
    model.count = modelData.indices.length;

    gl.bindBuffer(gl.ARRAY_BUFFER, model.coordsBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, modelData.vertexPositions, gl.STATIC_DRAW);
    gl.bindBuffer(gl.ARRAY_BUFFER, model.normalBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, modelData.vertexNormals, gl.STATIC_DRAW);
    gl.bindBuffer(gl.ARRAY_BUFFER, model.textureBuffer);
    gl.bufferData(gl.ARRAY_BUFFER, modelData.vertexTextureCoords, gl.STATIC_DRAW);

    console.log(modelData.vertexPositions.length);
    console.log(modelData.indices.length);

    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, model.indexBuffer);
    gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, modelData.indices, gl.STATIC_DRAW);


    model.render = function (prog) {
        gl.bindBuffer(gl.ARRAY_BUFFER, this.coordsBuffer);
        gl.vertexAttribPointer(prog.vcoordsLoc, 3, gl.FLOAT, false, 0, 0);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.normalBuffer);
        gl.vertexAttribPointer(prog.vnormalLoc, 3, gl.FLOAT, false, 0, 0);
        gl.bindBuffer(gl.ARRAY_BUFFER, this.textureBuffer);
        gl.vertexAttribPointer(prog.vtexcoordLoc, 2, gl.FLOAT, false, 0, 0);

        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.indexBuffer);

        gl.uniformMatrix4fv(prog.modelviewLoc, false, flatten(modelview));    //--- load flattened modelview matrix
        gl.uniformMatrix3fv(prog.normalmatrixLoc, false, flatten(normalmatrix));  //--- load flattened normal matrix

        gl.drawElements(gl.TRIANGLES, this.count, gl.UNSIGNED_SHORT, 0);
        console.log(this.count);
    }
    return model;
}



//  ######  Section added to allow shadow mapping (beginning)
function initFramebufferObject(gl) {
  var framebuffer, texture, depthBuffer;

  // Define the error handling function
  var error = function() {
    if (framebuffer) gl.deleteFramebuffer(framebuffer);
    if (texture) gl.deleteTexture(texture);
    if (depthBuffer) gl.deleteRenderbuffer(depthBuffer);
    return null;
  }

  // Create a framebuffer object (FBO)
  framebuffer = gl.createFramebuffer();
  if (!framebuffer) {
    console.log('Failed to create frame buffer object');
    return error();
  }

  // Create a texture object and set its size and parameters
  texture = gl.createTexture(); // Create a texture object
  if (!texture) {
    console.log('Failed to create texture object');
    return error();
  }
  gl.bindTexture(gl.TEXTURE_2D, texture);
  gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, OFFSCREEN_WIDTH, OFFSCREEN_HEIGHT, 0, gl.RGBA, gl.UNSIGNED_BYTE, null);
  gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);

  // Create a renderbuffer object and Set its size and parameters
  depthBuffer = gl.createRenderbuffer(); // Create a renderbuffer object
  if (!depthBuffer) {
    console.log('Failed to create renderbuffer object');
    return error();
  }
  gl.bindRenderbuffer(gl.RENDERBUFFER, depthBuffer);
  gl.renderbufferStorage(gl.RENDERBUFFER, gl.DEPTH_COMPONENT16, OFFSCREEN_WIDTH, OFFSCREEN_HEIGHT);

  // Attach the texture and the renderbuffer object to the FBO
  gl.bindFramebuffer(gl.FRAMEBUFFER, framebuffer);
  gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
  gl.framebufferRenderbuffer(gl.FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.RENDERBUFFER, depthBuffer);

  // Check if FBO is configured correctly
  var e = gl.checkFramebufferStatus(gl.FRAMEBUFFER);
  if (gl.FRAMEBUFFER_COMPLETE !== e) {
    console.log('Frame buffer object is incomplete: ' + e.toString());
    return error();
  }

  framebuffer.texture = texture; // keep the required object

  // Unbind the buffer object
  gl.bindFramebuffer(gl.FRAMEBUFFER, null);
  gl.bindTexture(gl.TEXTURE_2D, null);
  gl.bindRenderbuffer(gl.RENDERBUFFER, null);

  return framebuffer;
}
//  ######  End of section added to allow shadow mapping

function doKey(evt) {

    switch (evt.keyCode) {
        case 37: xdisplacement -= 1.0; break;        // left arrow
        case 39: xdisplacement += 1.0; break;       // right arrow
        case 38: ydisplacement += 1.0; break;        // up arrow
        case 40: ydisplacement -= 1.0; break;        // down arrow
    }
}
